///////////////////////////////////////////////////////////////////////////
//
//  ct_graphics.h   graphics routines to emulate CoreTools 
//                  Uses GLOBALs from ct_windows.h
//
////////////////////////////////////////////////////////////////////////////

#ifndef CT_GRAPHICS_H
#define CT_GRAPHICS_H

#include "ct_windows.h"

// plain screen point -- mirrors the functionality of struct POINT
// typedef struct tagPOINT {
//     LONG x;
//     LONG y; 
// } POINT; 

struct Point {
    int h;     // the x-coordinate
    int v;     // the y-coordinate 
    Point() : h(0), v(0) {}
    Point( int x_ , int y_ ) : h(x_), v(y_) {}
};

// Rectangle -- mirrors struct RECT of win32
// typedef struct _RECT {
//     LONG left;
//     LONG top;
//     LONG right;
//     LONG bottom;
// } RECT; 

struct Rect {
    int left;    // x-coordinate of the upper-left corner of the rectangle. 
    int top;     // y-coordinate of the upper-left corner of the rectangle
    int right;   // x-coordinate of the lower-right corner of the rectangle. 
    int bottom;  // y-coordinate of the lower-right corner of the rectangle. 

    Rect() : left(0), top(0), right(0), bottom(0) {}
    Rect( int l, int t, int r, int b ) : left(l), top(t), right(r), bottom(b) {}
    Rect( const Point& UL,  // upper-left corner   
          const Point& LR   // lower-right corner
          ) : left(UL.h), top(UL.v), right(LR.h), bottom(LR.v) {}
};

// Point and Rect manipulations

// Create or modify points

void SetPoint(Point& P, int x, int y);

void GetPoint(const Point& P, int& x, int& y);

Point MakePoint(int x, int y);

// Create or modify rectangles

void SetRect(Rect& R, int x1, int y1, int x2, int y2);

void GetRect(const Rect& R, int& x1, int& y1, int& x2, int& y2);

Rect MakeRect(int x1, int y1, int x2, int y2);

Rect MakeRect(Point UL, Point LR);

// Is the point inside the rectangle?

bool PointInRect(const Point& P, const Rect& R);

// MoveTo and LineTo to a screen point

void MoveTo( int x, int y );

void LineTo( int x, int y );

void MoveTo( const Point& P );

void LineTo( const Point& P );

// Move and Line move and draw a line segment RELATIVE to the current point

void Move( int x, int y );

void Line( int x, int y );

void DrawLine( int x1, int y1, int x2, int y2 );

void DrawLine( const Point& P1, const Point& P2 );

void PaintCircle( int x, int y, int r );

void PaintCircle( const Point& P, int r );

void FrameCircle( int x, int y, int r );

void FrameCircle( const Point& P, int r );

// "Painted" (solid) rectangle

void PaintRect( int x1, int y1, int x2, int y2 );

void PaintRect( const Point& UL, const Point& LR );

void PaintRect( const Rect& R ); 

// "Framed" (wire, empty) rectangle

void FrameRect( int x1, int y1, int x2, int y2 );

void FrameRect( const Point& UL, const Point& LR );

void FrameRect( const Rect& R ); 

// Win32 has a macro RGB with three parameters,
// RGB( color triple ) produces COLORREF, a 32-bit value.
// RGB is defined as 
// #define RGB(r, g ,b)  ((DWORD) (((BYTE) (r) | \
//     ((WORD) (g) << 8)) | \ 
//     (((DWORD) (BYTE) (b)) << 16))) 

void SetForeColor( COLORREF rgbc );  // allows SetForeColor( RGB( r, g, b ) );

void SetForeColor( int r, int g, int b );

void ClearDrawing();

#define GrayScale( s ) ( RGB(s,s,s) ) 
    
// RGB is defined as 
// #define RGB(r, g ,b)  ((DWORD) (((BYTE) (r) | \
//     ((WORD) (g) << 8)) | \ 
//     (((DWORD) (BYTE) (b)) << 16))) 

void SetColorPixel( int x, int y, short r, short g, short b );

void SetColorPixel( int x, int y, COLORREF rgbc );

void GetColorPixel( int x, int y, int& r, int& g, int& b );

#endif  // CT_GRAPHICS_H